package wechat

import (
	"encoding/json"
	"fmt"
	"time"

	"blog.bing89.com/go/wechat/curl"
)

type Token struct {
	Token     string `json:"access_token"`
	ExpiresIn int64  `json:"expires_in"`
	ExpiredAt int64  `json:"expiredAt"`
	ErrCode   int    `json:"errcode"`
	ErrMsg    string `json:"errmsg"`
}

type Wechat struct {
	AppID     string
	AppSecret string
	token     Token
}

func NewWechat(id, secret string) *Wechat {
	w := Wechat{
		AppID:     id,
		AppSecret: secret,
	}
	w.checkToken()
	return &w
}

func (w *Wechat) checkToken() {
	if w.token.Token == "" || w.token.ExpiredAt < time.Now().Unix() {
		w.getToken()
	}
}

func (w *Wechat) queryStr() string {
	return fmt.Sprintf("appid=%s&secret=%s", w.AppID, w.AppSecret)
}

func (w *Wechat)queryToken()string {
	w.checkToken()
	return fmt.Sprintf("access_token=%s", w.token.Token)
}

func (w *Wechat) getToken() error {
	url := fmt.Sprintf("%s??grant_type=client_credential&%s", APIGetAccessToken, w.queryStr())
	data, err := curl.SimpleGet(url)
	if err != nil {
		return err
	}
	err = json.Unmarshal(data, &w.token)
	if err == nil {
		w.token.ExpiredAt = w.token.ExpiresIn + time.Now().Unix()
	}
	return err
}

func (w *Wechat) SendText(to, text string) ([]byte, error) {
	msg := Message{
		ToUser: to,
		Type: MessageTypeText,
		Text: TextMessage{
			Content: text,
		},
	}
	url := fmt.Sprintf("%s?%s", APISendMessage, w.queryToken())
	data, err := curl.Post(url, &msg, nil)
	return data ,err
}
